package com.ozs.service.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.service.entity.BaseUser;
import com.ozs.service.entity.MsgAlarm;
import com.ozs.service.entity.vo.*;
import com.ozs.service.mapper.BaseCameraManagementMapper;
import com.ozs.service.mapper.BaseUserMapper;
import com.ozs.service.mapper.MsgAlarmMapper;
import com.ozs.service.service.MsgAlarmService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.*;

/**
 * <p>
 * 报警信息表 服务实现类
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@Service
public class MsgAlarmServiceImpl extends ServiceImpl<MsgAlarmMapper, MsgAlarm> implements MsgAlarmService {
    @Autowired
    private MsgAlarmMapper msgAlarmMapper;
    @Autowired
    private BaseCameraManagementMapper baseCameraManagementMapper;

    public IPage listToPage(List list, int pageNum, int pageSize) {
        List pageList = new ArrayList<>();
        int curIdx = pageNum > 1 ? (pageNum - 1) * pageSize : 0;
        for (int i = 0; i < pageSize && curIdx + i < list.size(); i++) {
            pageList.add(list.get(curIdx + i));
        }
        IPage page = new Page<>(pageNum, pageSize);
        page.setRecords(pageList);
        page.setTotal(list.size());
        return page;
    }

    //web数据统计 报警数据详情
    @Override
    public IPage<AlarmStatisticVo> list(AlarmStatisticResVo alarmStatisticResVo) {
        int pageNum = Integer.parseInt(alarmStatisticResVo.getPageNum().toString());
        int pageSize = Integer.parseInt(alarmStatisticResVo.getPageSize().toString());
        com.github.pagehelper.Page<AlarmStatisticVo> page = PageHelper.startPage(pageNum, pageSize)
                .doSelectPage(() -> msgAlarmMapper.list(alarmStatisticResVo));
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<AlarmStatisticVo> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(pageNum, pageSize);
        pageR.setRecords(page.getResult());
        pageR.setTotal(page.getTotal());
        return pageR;
    }

    @Override
    public List<AlarmStatisticMonthDto> getCount(AlarmStatisticResVo alarmStatisticResVo) {
        List<AlarmStatisticMonthDto> list = msgAlarmMapper.listDtoMonth(alarmStatisticResVo);
        return list;
    }

    @Override
    public List<AlarmStatisticMonthDto> getDayCount(AlarmStatisticResVo alarmStatisticResVo) {
        List<AlarmStatisticMonthDto> list = msgAlarmMapper.listDtoDay(alarmStatisticResVo);
        return list;
    }

    @Override
    public List<AlarmStatisticDto> exportExcel(AlarmStatisticResVo alarmStatisticResVo) {
        List<AlarmStatisticDto> listDto = msgAlarmMapper.listDto(alarmStatisticResVo);
        return listDto;
    }

    @Override
    public MsgAlarm selectByCameraCode(String cameraCode) {
        return msgAlarmMapper.selectByCameraCode(cameraCode);
    }

    @Override
    public IPage<MsgAlarm> queryPage(MsgAlarmVo vo) {
        int pageNum = Integer.parseInt(vo.getPageNum().toString());
        int pageSize = Integer.parseInt(vo.getPageSize().toString());
        com.github.pagehelper.Page<MsgAlarm> page = PageHelper.startPage(pageNum, pageSize)
                .doSelectPage(() -> msgAlarmMapper.selectMsgAlarmList(vo));
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<MsgAlarm> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(pageNum, pageSize);
        pageR.setRecords(page.getResult());
        pageR.setTotal(page.getTotal());
        return pageR;
    }

    @Override
    public MsgAlarm getByAlarmId(String alarmId) {
        return msgAlarmMapper.getByAlarmId(alarmId);
    }

    @Override
    public Map<String, Integer> statisticsNum(MsgAlarmVo msgAlarmVo) {
        Map<String, Integer> map = new HashMap<>();
        // 相机数量
        Integer countCamera = baseCameraManagementMapper.countCamera(msgAlarmVo);
        // 累计报警次数
        Integer countMsg = msgAlarmMapper.countMsg(msgAlarmVo);
        // 解除报警
//        LambdaQueryWrapper<MsgAlarm> lwJC = new LambdaQueryWrapper<MsgAlarm>();
//        lwJC.eq(MsgAlarm::getIsLock, "1");
        Integer countJC = msgAlarmMapper.countJC(msgAlarmVo);
        // 未解除报警
//        LambdaQueryWrapper<MsgAlarm> lwWJC = new LambdaQueryWrapper<MsgAlarm>();
//        lwWJC.eq(MsgAlarm::getIsLock, "2");
        Integer countWJC = msgAlarmMapper.countWJC(msgAlarmVo);
        map.put("countCamera", countCamera);
        map.put("countMsg", countMsg);
        map.put("countJC", countJC);
        map.put("countWJC", countWJC);
        return map;
    }

    @Override
    public Integer appAlarmNum(MsgAlarmVo msgAlarmVo) {
        return msgAlarmMapper.countWJC(msgAlarmVo);
    }

    @Override
    public List<MsgAlarm> msgAlarmList(MsgAlarmVo msgAlarmVo) {
        return msgAlarmMapper.selectMsgAlarmList(msgAlarmVo);
    }

    @Override
    public List<MsgAlarm> queryAlarmInfoByAfterTenMinute() {
        return msgAlarmMapper.queryAlarmInfoByAfterTenMinute();
    }

    @Override
    public IPage<MsgAlarm> queryPageHome(MsgAlarmVo msgAlarmVo) {
        int pageNum = Integer.parseInt(msgAlarmVo.getPageNum().toString());
        int pageSize = Integer.parseInt(msgAlarmVo.getPageSize().toString());
        com.github.pagehelper.Page<MsgAlarm> page = PageHelper.startPage(pageNum, pageSize)
                .doSelectPage(() -> msgAlarmMapper.selectMsgAlarmListHome(msgAlarmVo));
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<MsgAlarm> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(pageNum, pageSize);
        pageR.setRecords(page.getResult());
        pageR.setTotal(page.getTotal());
        return pageR;
    }
}
