package com.ozs.service.utils;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ozs.common.utils.sign.Md5Utils;
import com.ozs.common.utils.stateSecrets.SM4Utils;
import com.ozs.common.utils.uuid.IdUtils;
import com.ozs.service.entity.vo.BaseCameraVersionVo;
import com.ozs.service.entity.vo.Codec;
import com.ozs.service.entity.vo.Data;
import com.ozs.service.entity.vo.Heartbeat;
import com.ozs.service.entity.vo.Param;
import com.ozs.service.entity.vo.Svp;
import com.ozs.service.entity.vo.UpdateDeviceSn;
import com.ozs.service.entity.vo.Venc0;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.MqttPersistenceException;
import org.eclipse.paho.client.mqttv3.MqttTopic;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;


/**
 * 发布客户端
 *
 * @author Administrator
 */
@Slf4j
@Component
public class PublishClient {

    /**
     * mqtt服务器地址
     */
    public static String HOST;
    public static String USER_NAME;
    public static String PASS_WORD;
    private static String clientId;
    /**
     * 客户端实例
     */
    private static MqttClient client;
    @Autowired
    private RabbitMqConfig rabbitMqConfig;

    /**
     * 获取 MqttTopic
     */
    public static MqttTopic getMqttTopic(String topic) {
        return client.getTopic(topic);
    }

    /**
     * 发布
     *
     * @param topic
     * @param message
     * @throws MqttPersistenceException
     * @throws MqttException
     */
    public static void publish(MqttTopic topic, MqttMessage message) throws MqttPersistenceException,
            MqttException {
        MqttDeliveryToken token = topic.publish(message);
        token.waitForCompletion();
        System.out.println("message is published completely! " + token.isComplete());
    }

    public static void pull(String sign, String deviceSn) throws MqttException {
        /**
         * 发布客户端
         */
        Heartbeat test = new Heartbeat();
        test.setName("HeartResponse");
        Data data = new Data();
        data.setAlarm(0);
        data.setStream(0);
        test.setData(data);
        test.setCode(200);
        test.setSign(sign);
        String s = JSON.toJSONString(test);
        MqttMessage message = new MqttMessage();
        /**
         * 保证消息能到达一次
         */
        message.setQos(1);
        /**
         * 消息保留
         */
//        server.message.setRetained(false);
        /**
         * 消息内容
         */
        message.setPayload(s.getBytes());
        /**
         * 发布
         */
        publish(getMqttTopic("heart_" + deviceSn), message);
    }

    public static void updateDeviceSn(BaseCameraVersionVo baseCameraVersionVo) {
        /**
         * 发布客户端
         */
        log.info("updateDeviceSn---start");
        for (String code : baseCameraVersionVo.getCameraCodeList()) {
            try {
                log.info("update_" + code);
                UpdateDeviceSn updateDeviceSn = new UpdateDeviceSn();
                updateDeviceSn.setName("UpdateRequest");
                updateDeviceSn.setType(Integer.valueOf(baseCameraVersionVo.getUpgradeType()));
                updateDeviceSn.setMd5(baseCameraVersionVo.getMd5());
                updateDeviceSn.setSign(IdUtils.fastSimpleUUID());
                updateDeviceSn.setUrl(baseCameraVersionVo.getVersionAddress());
                String s = JSON.toJSONString(updateDeviceSn);
                MqttMessage message = new MqttMessage();
                /**
                 * 保证消息能到达一次
                 */
                message.setQos(1);
                /**
                 * 消息保留
                 */
//                server.message.setRetained(false);
                /**
                 * 消息内容
                 */
                message.setPayload(s.getBytes());
                /**
                 * 发布
                 */
                publish(getMqttTopic("update_" + code), message);
                log.info("updateDeviceSn---end");
            } catch (MqttException e) {
                System.out.println("updateDeviceSn-------" + e.getMessage());
            }
        }
    }

    public static void confidenceCoefficient(BaseCameraVersionVo baseCameraVersionVo, String value) {
        /**
         * 发布客户端
         */
        for (String code : baseCameraVersionVo.getCameraCodeList()) {
            try {
                JSONObject res = new JSONObject();
                res.put("Name", "ConfigRequest");
                Svp svp = new Svp();
                Param param = new Param();
                param.setThreshold(Double.valueOf(value));
                svp.setParam(param);
                Map<String, Object> map = new HashMap<>();
                map.put("svp", svp);
                res.put("data", map);
                res.put("sign", "cien" + IdUtils.fastSimpleUUID());
                String s = JSONObject.toJSONString(res);

                MqttMessage message = new MqttMessage();
                /**
                 * 保证消息能到达一次
                 */
                message.setQos(1);
                /**
                 * 消息保留
                 */
//               message.setRetained(false);
                /**
                 * 消息内容
                 */
                message.setPayload(s.getBytes());
                /**
                 * 发布
                 */
                publish(getMqttTopic("config_" + code), message);
            } catch (MqttException e) {
                System.out.println(e.getMessage());
            }
        }
    }

    public static void configFrameRate(BaseCameraVersionVo baseCameraVersionVo, String value) {
        /**
         * 发布客户端
         */
        for (String code : baseCameraVersionVo.getCameraCodeList()) {
            try {
                JSONObject res = new JSONObject();
                res.put("Name", "ConfigRequest");
                Codec codec = new Codec();
                Venc0 venc0 = new Venc0();
                venc0.setFps(Double.valueOf(value));
                codec.setVenc0(venc0);
                Map<String, Object> map = new HashMap<>();
                map.put("codec", codec);
                res.put("data", map);
                res.put("sign", "rate" + IdUtils.fastSimpleUUID());
                String s = JSONObject.toJSONString(res);

                MqttMessage message = new MqttMessage();
                /**
                 * 保证消息能到达一次
                 */
                message.setQos(1);
                /**
                 * 消息保留
                 */
//              message.setRetained(false);
                /**
                 * 消息内容
                 */
                message.setPayload(s.getBytes());
                /**
                 * 发布
                 */
                publish(getMqttTopic("config_" + code), message);
            } catch (MqttException e) {
                System.out.println(e.getMessage());
            }
        }
    }

    @PostConstruct
    public void init() {
        HOST = rabbitMqConfig.getHost();
        USER_NAME = rabbitMqConfig.getUserName();
        PASS_WORD = rabbitMqConfig.getPassword();
        clientId = rabbitMqConfig.getClientId();
        /**
         * 连接配置
         */
        MqttConnectOptions options = new MqttConnectOptions();
        /**
         * 不保存，每次重启新client
         */
        options.setCleanSession(true);
        options.setUserName(USER_NAME);
        options.setPassword(PASS_WORD.toCharArray());
        /**
         * 设置超时时间
         */
        options.setConnectionTimeout(10);
        /**
         * 设置会话心跳时间
         */
        options.setKeepAliveInterval(20);
        try {
            /**
             * 设置发布回调
             */
            client = new MqttClient(HOST, clientId, new MemoryPersistence());

            client.setCallback(new PublishCallback());
            client.connect(options);
            int[] Qos = {1};
            String[] topic1 = {"config", "update", "heart", "test"};
            client.subscribe(topic1);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
