package com.ozs.web.controller.monitor;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.annotation.Log;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseUser;
import com.ozs.service.entity.CameraLogDetail;
import com.ozs.service.entity.CameraUpdateLog;
import com.ozs.service.entity.vo.CameraUpdateLogResVo;
import com.ozs.service.entity.vo.CameraUpdateLogResp;
import com.ozs.service.service.BaseCameraManagementService;
import com.ozs.service.service.BaseUserService;
import com.ozs.service.service.CameraLogDetailService;
import com.ozs.service.service.CameraUpdateLogService;
import com.ozs.system.service.ISysDictDataService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author wyy
 * @subject
 * @creat 2023/4/24
 */
@RestController
@RequestMapping("/monitor/updateLog")
public class CameraUpdateLogController extends BaseController {
    private static final Logger log = LoggerFactory.getLogger(CameraUpdateLogController.class);
    @Autowired
    private CameraUpdateLogService cameraUpdateLogService;
    @Autowired
    private CameraLogDetailService cameraLogDetailService;
    @Resource
    BaseCameraManagementService baseCameraManagementService;
    @Autowired
    private ISysDictDataService dictDataService;
    @Resource
    private BaseUserService baseUserService;

    /**
     * 获取相机升级日志分页列表
     *
     * @param cameraUpdateLogResp
     * @return
     */
    @ApiOperation(value = "获取相机升级日志分页列表")
    @PostMapping("/list")
    public AjaxResult list(@RequestBody CameraUpdateLogResp cameraUpdateLogResp) throws ParseException {
        LambdaQueryWrapper<CameraUpdateLog> wrapper = new LambdaQueryWrapper<CameraUpdateLog>();
        if (!ObjectUtils.isEmpty(cameraUpdateLogResp.getVersionNum())) {
            wrapper.like(CameraUpdateLog::getVersionNum, cameraUpdateLogResp.getVersionNum());
        }
        if (!ObjectUtils.isEmpty(cameraUpdateLogResp.getUpgradeType())) {
            wrapper.like(CameraUpdateLog::getUpgradeType, cameraUpdateLogResp.getUpgradeType());
        }
        wrapper.orderByDesc(CameraUpdateLog::getCreateTime);
        IPage<CameraUpdateLog> page = cameraUpdateLogService.page(new Page<>(cameraUpdateLogResp.getPageNum(), cameraUpdateLogResp.getPageSize()), wrapper);
        List<CameraUpdateLog> list = cameraUpdateLogService.list();
        if (!ObjectUtils.isEmpty(page) && !ObjectUtils.isEmpty(page.getRecords())) {
            List<CameraUpdateLog> dto = page.getRecords().stream().map(o -> {
                for (CameraUpdateLog li : list) {
                    if (li.getUpgradeType().equals(o.getUpgradeType())) {
                        o.setUpgradeTypeName(dictDataService.selectDictLabel("upgrade_type", String.valueOf(li.getUpgradeType())));
                        BaseUser baseUser = baseUserService.getUser(o.getCreateBy());
                        o.setCreateByName(baseUser.getUserName());
                    }
                }
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto);
        }
        return AjaxResult.success(page);
    }

    @ApiOperation(value = "获取相机升级日志详细")
    @PostMapping("/detail/{pageNum}/{pageSize}")
    public AjaxResult detail(@RequestBody CameraUpdateLogResVo cameraUpdateLogResVo,@PathVariable Integer pageNum,@PathVariable Integer pageSize) {
        List<CameraLogDetail> list = cameraLogDetailService.selectDetail((cameraUpdateLogResVo));
        list.forEach(li -> {
            LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<BaseCameraManagement>();
            wrapper.eq(!ObjectUtils.isEmpty(li.getCameraCode()),BaseCameraManagement::getCameraCode,li.getCameraCode());
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(wrapper);
            li.setLastVersionNum(baseCameraManagement.getLastVersionNum());
        });
        IPage<CameraLogDetail> page = cameraLogDetailService.listToPage(list, pageNum, pageSize);
        return AjaxResult.success(page);
    }

    @Log(title = "相机升级日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody CameraUpdateLogResVo cameraUpdateLogResVo) {
        List<CameraLogDetail> list = cameraLogDetailService.selectDetail(cameraUpdateLogResVo);
        list.forEach(li -> {
            LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<BaseCameraManagement>();
            wrapper.eq(!ObjectUtils.isEmpty(li.getCameraCode()),BaseCameraManagement::getCameraCode,li.getCameraCode());
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(wrapper);
            li.setLastVersionNum(baseCameraManagement.getLastVersionNum());
        });
        ExcelUtil<CameraLogDetail> util = new ExcelUtil<CameraLogDetail>(CameraLogDetail.class);
        util.exportExcel(response, list, "相机升级日志详情");
    }
}
