package com.ozs.web.controller.shotschedule;

import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.utils.DictUtils;
import com.ozs.system.service.ISysDictDataService;
import com.ozs.system.service.ISysDictTypeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * @author wyy
 * @subject
 * @creat 2023/5/6
 */
@Configuration
@EnableScheduling
@EnableAsync
@Slf4j
public class ShotExecutors {

   // private  static ScheduledExecutorService scheduledExecutorService = Executors.newScheduledThreadPool(5);
    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private ISysDictTypeService iSysDictTypeService;
    @Autowired
    private TaskNormalService taskNormalService; //定时任务-没有报警摄像头
    @Autowired
    private TaskAlarmService taskAlarmService;//定时任务-报警摄像头

    /**
     * 处理非预警摄像头截图
     */
    @Async
    @Scheduled(fixedRate = 60000)
    public void handleNormalPicture() {
        List<SysDictData> dataList = DictUtils.getDictCache("shot_switch");
        if(CollectionUtils.isEmpty(dataList) || Objects.isNull(dataList.get(0))){
            dataList = iSysDictTypeService.selectDictDataByType("shot_switch");
            DictUtils.setDictCache("shot_switch",dataList);
        }
        //boolean shotSwitch = Boolean.parseBoolean(dictDataService.selectDictLabel("shot_switch", String.valueOf(0)));
        try {
            if(!CollectionUtils.isEmpty(dataList) && Objects.nonNull(dataList.get(0))){
                if (Boolean.parseBoolean(dataList.get(0).getDictLabel())) {
                    taskNormalService.getNormalPicture();
                }
            }
        } catch (Exception e) {
            e.getMessage();
            log.error(e.getMessage());
        }
    }

    /**
     * 处理预警摄像头截图
     */
    @Async
    @Scheduled(fixedRate = 500)
    public void handleAlarmPicture() {
        List<SysDictData> dataList = DictUtils.getDictCache("shot_switch");
        if(CollectionUtils.isEmpty(dataList) || Objects.isNull(dataList.get(0))){
            dataList = iSysDictTypeService.selectDictDataByType("shot_switch");
            DictUtils.setDictCache("shot_switch",dataList);
        }
        //boolean shotSwitch = Boolean.parseBoolean(dictDataService.selectDictLabel("shot_switch", String.valueOf(0)));
        try {
            if(!CollectionUtils.isEmpty(dataList) && Objects.nonNull(dataList.get(0))){
                if (Boolean.parseBoolean(dataList.get(0).getDictLabel())) {
                    taskAlarmService.getAlarmPicture();
                }
            }
        } catch (Exception e) {
            e.getMessage();
            log.error(e.getMessage());
        }
    }
}
