package com.ozs.web.controller.websocket;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.service.entity.*;
import com.ozs.service.entity.vo.MsgAlarmResp;
import com.ozs.service.service.*;
import com.ozs.system.service.ISysDictDataService;
import com.ozs.system.service.ISysUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.websocket.Session;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author : sunhh
 * @create 2023/3/13 22:08
 */
@Slf4j
@Controller
@RequestMapping("/sdk")
public class WebSocketConteoller extends BaseController {

    @Autowired
    private WebSocketServer webSocketServer;
    @Autowired
    private BaseUserService baseUserService;
    @Autowired
    private ISysUserService userService;
    @Resource
    private BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private BaseCameraManagementService baseCameraManagementService;
    @Autowired
    private UserRegistrationidService userRegistrationidService;
    @Autowired
    private MsgWebPushService msgWebPushService;
    @Autowired
    private MsgAppPushService msgAppPushService;
    @Autowired
    private ISysDictDataService dictDataService;

    /**
     * 获取项目名称
     */
    @ResponseBody
    @GetMapping("/projectName")
    public String projectName() {
        String projectName = dictDataService.selectDictLabel("sys_name", "name");
//        HashMap<String, Object> map = new HashMap<>();
//        map.put("name", projectName);
        return projectName;
    }

    /**
     * app 登陆时查询 未登陆时推送的消息 重新推送
     * @return
     */
    @ResponseBody
    @GetMapping("/getMessageApp")
    public Map publishApp() {
        String userId = getUserId();
        List<MsgAppPush> msgAppPushList = msgAppPushService.getByUserId(userId);
        for (MsgAppPush msgAppPush : msgAppPushList) {
            String remark = msgAppPush.getRemark();
            MsgAlarmResp msgAlarmResp = JSONObject.parseObject(remark, MsgAlarmResp.class);
            Date alarmTime = msgAlarmResp.getAlarmTime();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String formatDate = sdf.format(alarmTime != null ? alarmTime : new Date());
            List<String> userList = new ArrayList<>();
            userList.add(userId);
            JPushUtil.sendToRegistrationId(
                    userList,
                    "报警通知！", msgAlarmResp.getRailwayName() != null ? msgAlarmResp.getRailwayName() : "",
                    // content,
                    formatDate + "     " + (msgAlarmResp.getAlarmType() == 1 ? "泥石流" : msgAlarmResp.getAlarmType()) + "     " + msgAlarmResp.getLineDir() + "     " + msgAlarmResp.getAlarmMile() + "     " + msgAlarmResp.getBeginMile(),
                    remark);
            msgAppPush.setOnline("1");
            msgAppPushService.updateById(msgAppPush);
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("code", 200);
        return map;
    }

    /**
     * web 登陆时查询 未登陆时推送的消息 重新推送
     * @return
     */
    @ResponseBody
    @GetMapping("/getMessage")
    public Map publishWeb() {
        String userId = getUserId();
        List<MsgWebPush> msgWebPushList = msgWebPushService.getByUserId(userId);
        for (MsgWebPush msgWebPush : msgWebPushList) {
            webSocketServer.sendOneMessage(userId, msgWebPush.getRemark());
            msgWebPush.setOnline("1");
            msgWebPushService.updateById(msgWebPush);
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("code", 200);
        return map;
    }

    /**
     * 推送消息
     * @param msgAlarm
     * @return
     */
    @ResponseBody
    @PostMapping("/publish")
    public Map publish(@RequestBody MsgAlarm msgAlarm) {
        String endMile = null;
        log.info("消息推送:" + msgAlarm.toString());
        LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
            wrapper.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(wrapper);
        log.info("baseCameraManagement-----" + baseCameraManagement);
        // 获取推送人ID列表
        log.info("publish-----" + baseCameraManagement.getCreateBy() + "-------------" + baseCameraManagement.getDeptId());
        Set<String> set = baseUserService.getUserIdList(baseCameraManagement.getCreateBy(), baseCameraManagement.getDeptId());
        log.info("set-----" + set);
        String[] list = set.toArray(new String[set.size()]);
        List<String> userIdList = new ArrayList<>(set);
        MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile())&&!ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            endMile = begin + "-" + end;
            msgAlarmResp.setBeginMile(endMile);
        }
        msgAlarmResp.setAlarmType(msgAlarm.getAlarmType());
        String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
        msgAlarmResp.setAlarmMile(mile);
        String cameraCode = baseCameraManagement.getCameraCode();
        msgAlarmResp.setCameraCode(cameraCode);
        
        String cameraName = baseRailwayManagementService.getCameraNameByCameraCode(cameraCode);
        msgAlarmResp.setRailwayName(cameraName != null ? cameraName : "");
        msgAlarmResp.setLineDirStr(baseCameraManagement.getLineDir() == 1 ? "上行" : "下行");
        msgAlarmResp.setAlarmTime(msgAlarm.getAlarmTime() != null ? msgAlarm.getAlarmTime() : new Date());
        // 图片
        if (!StringUtils.isEmpty(msgAlarm.getImageUrl())) {
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            log.info("publish--------" + objects);
            msgAlarmResp.setImageUrls(objects);
        }
        if (!StringUtils.isEmpty(msgAlarm.getId())) {
            msgAlarmResp.setId(msgAlarm.getId());
        }
        String userIdLogin = "admin编码";
        SysUser user = userService.selectUserById(1L);
        if (!ObjectUtils.isEmpty(user)) {
            userIdLogin = user.getUserId();
        }
        String alarmId = msgAlarm.getAlarmId();
        msgAlarmResp.setAlarmId(alarmId);

        LambdaQueryWrapper<MsgWebPush> lwMsgWebPush = new LambdaQueryWrapper<MsgWebPush>();
        if (!ObjectUtils.isEmpty(msgAlarm.getAlarmId())) {
            lwMsgWebPush.eq(MsgWebPush::getAlarmId, msgAlarm.getAlarmId());
        }
        List<MsgWebPush> msgWebPushList = msgWebPushService.list(lwMsgWebPush);
        if (StringUtils.isEmpty(msgWebPushList) || msgWebPushList.size() == 0) {
            for (String userid : list) {
                String pushId = UUID.randomUUID().toString();
                msgAlarmResp.setPushId(pushId);
                String content = objStr(msgAlarmResp);
                log.info("WEBcontent-----" + content);
                log.info("WEBuserid-----" + userid);
                // 存入数据
                MsgWebPush msgWebPush = new MsgWebPush();
                // 推送消息reqMsgAlarmVo
                boolean userOnline = webSocketServer.userOnline(userid);
                log.info("userOnline-userOnline-userOnline-userOnline-----：：：：：：" + userOnline);
                if (userOnline) {
                    webSocketServer.sendOneMessage(userid, content);
                    msgWebPush.setOnline("1");
                } else {
                    // online 是否推送，1:推送，2:未推
                    msgWebPush.setOnline("2");
                }
                msgWebPush.setRemark(content);
                // 状态：1未读；2已读
                msgWebPush.setStatus(1);
                msgWebPush.setPushId(pushId);
                msgWebPush.setAlarmId(alarmId);
                msgWebPush.setReceiveBy(userid);
                msgWebPush.setCreateBy(userIdLogin);
                msgWebPush.setCreateTime(new Date());
                msgWebPush.setUpdateBy(userIdLogin);
                msgWebPush.setUpdateTime(new Date());
                msgWebPushService.save(msgWebPush);
            }
        }

        // app 消息推送
        LambdaQueryWrapper<MsgAppPush> lwMsgAppPush = new LambdaQueryWrapper<MsgAppPush>();
        if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
            lwMsgAppPush.eq(MsgAppPush::getAlarmId, msgAlarm.getAlarmId());
        }
        List<MsgAppPush> msgAppPushList = msgAppPushService.list(lwMsgAppPush);
        log.info("APPmsgAppPushList     size -----" + msgAppPushList.size());
        if (StringUtils.isEmpty(msgAppPushList) || msgAppPushList.size() == 0) {
            List<UserRegistrationid> alias = new ArrayList<>();
            if (!StringUtils.isEmpty(userIdList) || userIdList.size() > 0) {
                alias = userRegistrationidService.selectRegistrationID(userIdList);
            }
//            List<String> listString = distinct(alias);
            if (!StringUtils.isEmpty(alias) || alias.size() > 0) {
                for (UserRegistrationid userRegistrationid : alias) {
                    log.info("APP  alias    -----" + userRegistrationid.getRegistrationId());
                    String pushId = UUID.randomUUID().toString();
                    msgAlarmResp.setPushId(UUID.randomUUID().toString());
                    String content = objStr(msgAlarmResp);
                    Date alarmTime = msgAlarm.getAlarmTime();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    String formatDate = sdf.format(alarmTime != null ? alarmTime : new Date());
                    log.info("APPcontent-----" + content);
                    log.info("APPuserid-----" + userRegistrationid.getUserId());
                    // 存入数据
                    MsgAppPush msgAppPush = new MsgAppPush();
                    // 推送消息reqMsgAlarmVo
                    boolean userOnline = webSocketServer.userOnline(userRegistrationid.getUserId());
                    log.info("userOnline-userOnline-userOnline-userOnline-----：：：：：：" + userOnline);
                    if (userOnline) {
                        List<String> userList = new ArrayList<>();
                        userList.add(userRegistrationid.getRegistrationId());
                        JPushUtil.sendToRegistrationId(
                                userList,
                                "报警通知！", cameraName != null ? cameraName : "",
                                // content,
                                formatDate + "     " + (msgAlarmResp.getAlarmType() == 1 ? "泥石流" : msgAlarmResp.getAlarmType()) + "     " + (baseCameraManagement.getLineDir() == 1 ? "上行" : "下行") + "     " + mile + "     " + endMile,
                                content);
                        msgAppPush.setOnline("1");
                    } else {
                        // online 是否推送，1:推送，2:未推
                        msgAppPush.setOnline("2");
                    }
                    msgAppPush.setRemark(content);
                    // 状态：1未读；2已读
                    msgAppPush.setStatus(1);
                    msgAppPush.setPushId(pushId);
                    msgAppPush.setAlarmId(alarmId);
                    msgAppPush.setReceiveBy(userRegistrationid.getUserId());
                    msgAppPush.setRegistrationId(userRegistrationid.getRegistrationId());
                    msgAppPush.setCreateBy(userIdLogin);
                    msgAppPush.setCreateTime(new Date());
                    msgAppPush.setUpdateBy(userIdLogin);
                    msgAppPush.setUpdateTime(new Date());
                    msgAppPushService.save(msgAppPush);
                }
            }
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("code", 200);
        return map;
    }

    /**
     * 去除重复数据
     * 由于Set的无序性，不会保持原来顺序
     *
     * @param list
     */
    public static List<String> distinct(List<String> list) {
        final boolean sta = null != list && list.size() > 0;
        List doubleList = new ArrayList();
        if (sta) {
            Set set = new HashSet();
            set.addAll(list);
            doubleList.addAll(set);
        }
        return doubleList;
    }

    private String objStr(MsgAlarmResp msgAlarmResp) {
        JSONObject obj = new JSONObject();
        obj.put("data", msgAlarmResp); //data
        obj.put("code", 110); //code
        log.info("【websocket消息】 单点消息:" + obj.toJSONString());
        return obj.toJSONString();
    }
}
